#!/usr/bin/perl
package NorthStar;

# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
# All rights reserved.
# 
# This file is part of NorthStar and is covered under
# the terms of the Apache Software License version 1.1
#
# A copy of this license is included in the LICENSE file
#
# $Header: /cvsroot/NorthStar/NorthStar.cgi,v 1.41 2002/03/29 04:35:57 brownkid Exp $
#

use DBI;
use CGI qw(-no_debug :standard);

$NorthStar::VERSION = "1.6";

my(@searchdir) = qw(/opt/NorthStar /usr/NorthStar /usr/local/NorthStar .);
my($t);

foreach(@searchdir) {
    if(-f "$_/NorthStar.conf") {
        ConfigInit("$_/NorthStar.conf");
        $t = 1;
        last;
    }
}

if(!$t) {
    die("Unable to find the configuration file.\n");
}

my(%config) = ConfigGetSection('Global');
my(%dbconfig) = ConfigGetSection('Database');
my($lib_dir) = $config{'InstallDir'} . "/lib/";

require $lib_dir . 'util.lib';
require $lib_dir . 'dbutil.lib';
require $lib_dir . 'search.lib';
require $lib_dir . 'location.lib';
require $lib_dir . 'owner.lib';
require $lib_dir . 'device.lib';
require $lib_dir . 'network.lib';
require $lib_dir . 'tree.lib';
require $lib_dir . 'user.lib';
require $lib_dir . 'session.lib';
require $lib_dir . 'permissions.lib';
require $lib_dir . 'template.lib';
require $lib_dir . 'regex.lib';
require $lib_dir . 'audit.lib';
require $lib_dir . 'view.lib';

my(%dbinfo) = (
  'fh'      => 'db',		              # Database FH, DONT change this
  'driver'	=> $dbconfig{'DatabaseDriver'},# DBI database driver
  'host'    => $dbconfig{'DatabaseHost'}, # The host address of your db server
  'port'    => $dbconfig{'DatabasePort'}, # Database port
  'options' => '',		                  # Unused
  'tty'     => '',		                  # Unused
  'dbname'  => $dbconfig{'DatabaseName'}, # The database to connect to
  'dbuser'  => $dbconfig{'DatabaseUser'}, # The user to connect as
  'dbpass'  => $dbconfig{'DatabasePass'}  # The password to use
);

$cgi = new CGI;
%form = $cgi->Vars;
$| = 1;

main();

sub main {
	my($sth);

	if(db_connect(\%dbinfo) != 0) {
		ExitError("db_connect: " . $DBI::errstr);
	}

    my($sid) = $cgi->cookie('NorthStarSID');

    if($form{'r'} eq 'logout') {
        my($c) = $cgi->cookie(-name  => 'NorthStarSID',
                              -value => ''
                             );
                              
        OutputHeader(-type  => 'text/html',
                     -cookie=>$c,
                     ); 

        $NorthStar::username = SessionInit($sid, $ENV{'REMOTE_ADDR'});
        SessionDelete($sid);
        Audit('session','logout',"session $sid logged out");
        my(%pitems) = ( 'msg' => 'You are logged out.  Have a nice day!' );
        OutputPage('login_page.html', \%pitems);
        exit(0);
    } elsif($sid) {
        $NorthStar::username = SessionInit($sid, $ENV{'REMOTE_ADDR'});
    } elsif($form{'username'} && $form{'passwd'}) {
        $form{'username'} = lc($form{'username'});
        SessionCreate($form{'username'}, $form{'passwd'}, $ENV{'REMOTE_ADDR'});
        $NorthStar::username = $form{'username'};
        Audit('session','login',"user $form{'username'} ($ENV{'REMOTE_ADDR'}) logged in");
    } else {
        OutputHeader();
        OutputPage('login_page.html');
        exit(0);
    }

    OutputHeader();

	$sth = $db->prepare("SELECT id,realname FROM users WHERE username = '" .
		                $NorthStar::username . "'");

	$sth->execute();
	db_rescheck(\$sth,'get_uid','1');
	($NorthStar::user_id, $NorthStar::user_realname) = $sth->fetchrow_array;
  
	VoodooMagic();
    undef($sth);
	$db->disconnect;
}

sub VoodooMagic {
	my(%routines) = (
		'shmain'    => 'ShowMain',
		'addloc'    => 'AddLocation',
		'updloc'    => 'UpdLocation',
		'delloc'    => 'DelLocation',
		'shloc'     => 'ShowLocation',
        'shloctree'  => 'ShowLocationTree',
        'addowner'  => 'AddOwner',
		'updowner'  => 'UpdOwner',
		'delowner'  => 'DelOwner',
		'showner'   => 'ShowOwner',
        'shownertree'  => 'ShowOwnerTree',
        'adddev'    => 'AddDevice',
		'upddev'    => 'UpdDevice',
		'deldev'    => 'DelDevice',
		'shdev'	    => 'ShowDevice',
        'shdevtree'  => 'ShowDeviceTree',
        'addnet'    => 'AddNet',
        'addnetrange' => 'AddNetRange',
		'updnet'    => 'UpdNet',
		'delnet'    => 'DelNet',
		'shnet'     => 'ShowNet',
        'shnettree'  => 'ShowNetTree',
        'addaddr'   => 'AddNet',
		'addaddrrange'=>'AddNetRange',
        'adduser'   => 'AddUser',
        'upduser'   => 'UpdUser',
        'deluser'   => 'DelUser',
        'shuser'    => 'ShowUser',
        'shusertree'=> 'ShowUserTree',
        'shtmpl'    => 'ShowTmpl',
        'search'    => 'ShowSearch',
        'changepass'=> 'UserChangePass',
        'addtemplate'=>'AddTemplate',
        'updtemplate'=>'UpdTemplate',
        'deltemplate'=>'DelTemplate',
        'shtemplate'=> 'ShowTemplate',
        'shtemplatetree'=>'ShowTemplateTree',
        'addfield'  => 'TemplateFieldAdd',
        'updfield'  => 'TemplateFieldUpdate',
        'delfield'  => 'TemplateFieldDelete',
        'shfield'   => 'ShowTemplateField',
        'shregex'   => 'ShowRegex',
        'shregextree'=>'ShowRegexTree',
        'addregex'  => 'AddRegex',
        'updregex'  => 'UpdRegex',
        'delregex'  => 'DelRegex',
        'shaudit'   => 'ShowAudit',
        'shviewtree'=> 'ShowViewTree',
        'shview'    => 'ShowView',
        'addview'   => 'AddView',
        'updview'   => 'UpdView',
        'delview'   => 'DelView',
        'allocwiz'  => 'NetworkAlloc'
	);


	my($sth) = $db->prepare("SELECT permmask FROM users WHERE id = '$user_id'");
	$sth->execute;
	db_rescheck(\$sth,'permmask_get','1');
    my($pm) = ($sth->fetchrow_array)[0];
    PermissionsInit($pm);
	if($form{'r'} && $routines{$form{'r'}}) {
		my($r) = $routines{$form{'r'}};
        if(PermissionsCheckPerm($form{'r'}) != 1) {
            ExitError("You do not have permission to execute this function");
        }

        &$r;
	} else {
		ShowMain();
	}
}

sub ConfigGetSection {
        my $sec = shift;
        my(%t);
        
        my($h) = $NorthStar::CONFIG{$sec};
        while(my($k,$v) = each %$h) {
                $t{$k} = $v;
        }
        return(%t);
}

sub ConfigGetValue {
        if(scalar(@_) == 3) {
                return($NorthStar::CONFIG{$_[0]}{$_[1]}{$_[2]});
        } elsif(scalar(@_) == 2) {
                return($NorthStar::CONFIG{$_[0]}{$_[1]});
        } else {
                return(undef);
        }
}

sub ConfigInit {
        my $cf = shift;
        my($cs, $csv, $cn, $cv, @j, $x);
        open(_CONFIG, "<$cf") || die("config file open failed: $cf: $!\n");
        while(<_CONFIG>) {
                chomp;
                s/^[\s\t]+//g; #strip leading whitespace
                next if /^#/;
                next if /^$/;
                s/\s+$//g; #strip trailing whitespace

                if(/\#/) {
                    /(.*)\#.*/;
                    $x = $1;
                } else {
                    $x = $_;
                }

                # given:
                # <Section SectionVariable>
                #                Name = Value
                # </Section>
                # $cs  = Section                Config Section        
                # $csv = SectionVariable        Config Section Variable 
                # $cn  = Name                   Config Name
                # $cv  = Value                  Config Variable
                # 
                # $csv may not be defined if a SectionVariable was not present
                if($x =~ /^<(.*)>$/ && $x !~ /^<\/.*>/) {           # handler for section starts <Section>
                        if(!defined($cs)) {              # check to see if the previous section was closed with </Section> 
                                $cs = $1;
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "previous configuration section not closed (maybe you forgot </$cs>)\n");
                        }

                        # sub-handler for sections with a variable <Section SectionVariable>
                        if($x =~ /^<(.*) (.*)>$/ && (@j = split(/\s+/, $x)) == 2) {
                                $cs = $1;
                                $csv = $2;
                        } elsif($x =~ /^<(.*)>$/) { # dont clobber sections with no SectionVariable (<Section> ..)
                                $cs = $1;
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "configuration sections may only have one parameter (<Section Parameter>)\n");
                        }
                        next;
                } elsif($x =~ /^<\/(.*)>$/) {            #handler for section closes </Section>
                        if(defined($cs)) {         #check to see if this section was ever opened
                                undef($cs);
                                undef($csv);
                        } else {
                                $cs = $1;          #set section name for error message
                                $cs =~ s/\///g;    #strip leading '/'
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "configuration section not opened (maybe you forgot <$cs>)\n");
                        }
                        next;
                } else {
                        if($x =~ /(.*) \"(.*)\"/) {               #process line "Name = "Value In Here"
                                $cn = $1;                                
                                $cv = $2;
                                $cv =~ s/\\(.)/$1/g;              #normalize escaped charecters like: \"
                        } elsif((@j = split(/\s+/, $x)) > 2) {    #process line "Name = Value1 Value2..."
                                $cn = shift(@j);
                                $cv = join(',', @j);              #make a comma delimited list
                        } elsif((@j = split(/\s+/, $x)) == 2) {   #process line "Name = Value"
                                $cn = $j[0];
                                $cv = $j[1];
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.\n");
                        }

                        if($csv) { #if SectionVariable was set this as CONFIG->Section->SectionVariable->Name=Value
                                #$self->Debug("add value: CONFIG->$cs->$csv->$cn=$cv");
                                $NorthStar::CONFIG{$cs}{$csv}{$cn} = $cv;
                        } else {   #else set this value under CONFIG->Section->Name=Value
                                #$self->Debug("add value: CONFIG->$cs->$cn=$cv");
                                $NorthStar::CONFIG{$cs}{$cn} = $cv;
                        }
                        
                        next;
                }
                
                
        }
        #$self->Debug("closing config file");
        close(_CONFIG) || die("close failed: $cf: $!\n");

        return 1;
}

sub ConfigPrint {
        if(!defined(%NorthStar::CONFIG)) {
                return;
        }
        
        while(my($k,$v) = each %NorthStar::CONFIG) {
                print STDERR "$k\n";
                while(my($y,$z) = each %$v) {
                        if(ref($z) eq "HASH") {
                                print STDERR " $y\n";
                                while(my($a,$b) = each %$z) {
                                        print STDERR "  $a=$b\n";
                                }
                        } else {
                                print STDERR " $y=$z\n";
                        }
                }
                print STDERR "\n";
        }
}
