-- 
-- Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>.
-- All rights reserved.
-- 
-- This file is part of NorthStar and is covered under
-- the terms of the Apache Software License version 1.1
--
-- A copy of this license is included in the LICENSE file
--
-- $Header: /cvsroot/NorthStar/NorthStar.postgres.sql,v 1.12 2002/03/29 07:33:22 brownkid Exp $
--


\c template1
CREATE USER northstar;
CREATE DATABASE northstar;
\c northstar northstar
CREATE SEQUENCE users_id_seq;
CREATE TABLE users (
	id		INT4 NOT NULL DEFAULT nextval('users_id_seq') PRIMARY KEY,  	
	username	VARCHAR(32) NOT NULL UNIQUE,
	passwd		VARCHAR(32) NOT NULL,
	realname	VARCHAR(128) NOT NULL,
	enabled		BOOL DEFAULT 'T',
	permmask	VARCHAR(128)
);
GRANT ALL ON users TO northstar;

CREATE SEQUENCE owners_id_seq;
CREATE TABLE owners (
	id              INT4 NOT NULL DEFAULT nextval('owners_id_seq') PRIMARY KEY,  	
	templateid	INT4 NOT NULL DEFAULT '1',
	name		VARCHAR(256) NOT NULL,
	descr		TEXT
);
GRANT ALL ON owners TO northstar;

CREATE SEQUENCE locations_id_seq;
CREATE TABLE locations (
	id              INT4 NOT NULL DEFAULT nextval('locations_id_seq') PRIMARY KEY,    
	parent		INT4 NOT NULL REFERENCES locations(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE DEFAULT '0',	
	templateid	INT4 NOT NULL DEFAULT '2',
	name		VARCHAR(256) NOT NULL,
	descr		TEXT,
	assignuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	assigndate	TIMESTAMP DEFAULT now(),
	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	upddate		TIMESTAMP DEFAULT now()
);
GRANT ALL ON locations TO northstar;

CREATE SEQUENCE devices_id_seq;
CREATE TABLE devices (
	id		INT4 NOT NULL DEFAULT nextval('devices_id_seq') PRIMARY KEY, 
	parent		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE DEFAULT '0',	
	templateid	INT4 NOT NULL DEFAULT '3',
	name		VARCHAR(256) NOT NULL,
	descr		TEXT,
	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE, 
	location	INT4 NOT NULL REFERENCES locations(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE, 
	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	assigndate	TIMESTAMP DEFAULT now(),
	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	upddate		TIMESTAMP DEFAULT now()
);
GRANT ALL ON devices TO northstar;

CREATE SEQUENCE networks_id_seq;
CREATE TABLE networks (
	id		INT4 NOT NULL DEFAULT nextval('networks_id_seq') PRIMARY KEY,
	parent		INT4 NOT NULL REFERENCES networks(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE DEFAULT '0',
	templateid	INT4 NOT NULL DEFAULT '4',
	viewid		INT4 NOT NULL,
	status		INT4 NOT NULL,
	name		VARCHAR(256) NOT NULL,
	descr		TEXT,
	label		VARCHAR(256) NOT NULL,
	network		NUMERIC(40,0) NOT NULL,
	masklen		INT4 NOT NULL,
	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,
	device		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,
	location	INT4 NOT NULL REFERENCES locations(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,
	priority	INT4 NOT NULL DEFAULT '5',
	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	assigndate	TIMESTAMP DEFAULT now(),
	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,
	upddate		TIMESTAMP DEFAULT now()
);
GRANT ALL ON networks TO northstar;

CREATE TABLE sessions (
	id		VARCHAR(32) NOT NULL PRIMARY KEY,
	date		NUMERIC(40,0),
	username	VARCHAR(32) NOT NULL,
	ip		VARCHAR(20)
);
GRANT ALL ON sessions TO northstar;

CREATE SEQUENCE templates_id_seq START 6;
CREATE TABLE templates (
	id		INT4 NOT NULL DEFAULT nextval('templates_id_seq') PRIMARY KEY,
	objtype		INT2,
	name		VARCHAR(64) NOT NULL,
	descr		TEXT
);
GRANT ALL ON templates TO northstar;

CREATE SEQUENCE fielddefs_id_seq START 16;
CREATE TABLE fielddefs (
	id 		INT4 NOT NULL DEFAULT nextval('fielddefs_id_seq') PRIMARY KEY,
	templateid	INT4,
	name		VARCHAR(64),
	descr		TEXT,
	required	BOOL DEFAULT 'F',
	regex		INT4,
	minlength	INT4,
	maxlength	INT4,
	sortkey		INT4,
	fieldtype	VARCHAR(32),
	fieldopt	VARCHAR(255),
	updatemode	INT2
);
GRANT ALL ON fielddefs TO northstar;

CREATE SEQUENCE regex_id_seq START 15;
CREATE TABLE regex (
	id		INT4 NOT NULL DEFAULT nextval('regex_id_seq') PRIMARY KEY,
	name		VARCHAR(64),
	example		VARCHAR(64),
	regex		VARCHAR(255)
);
GRANT ALL ON regex TO northstar;

CREATE TABLE fielddata (
	id		INT4 NOT NULL,
	objtype		INT4 NOT NULL,
	fieldid		INT4 NOT NULL,
	data		TEXT
);
GRANT ALL ON fielddata TO northstar;	

CREATE SEQUENCE auditlog_id_seq;
CREATE TABLE auditlog (
	id		INT4 NOT NULL DEFAULT nextval('auditlog_id_seq') PRIMARY KEY,
	date		TIMESTAMP DEFAULT NOW(),
	username	VARCHAR(64),
	section		VARCHAR(25),
	action		VARCHAR(25),
	msg		TEXT
);
GRANT ALL ON auditlog TO northstar;

CREATE SEQUENCE views_id_seq START 2;
CREATE TABLE views (
	id		INT4 NOT NULL DEFAULT nextval('views_id_seq') PRIMARY KEY,
	name		VARCHAR(64),
	descr		TEXT
);
GRANT ALL ON views TO northstar;

INSERT INTO users VALUES (0,'admin','ekGaihSJjmIL.','Administrator','T','11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111');
INSERT INTO locations VALUES('0','0','2','Default Location','Default Location','0','now','0','now');
INSERT INTO owners VALUES('0','1','Default Owner','Default Owner');
INSERT INTO devices VALUES('0','0','3','Default Device','Default Device','0','0','0','now','0','now');
INSERT INTO networks VALUES('0','0','4','0','1','Default Network','Default Network','0.0.0.0/0','0','0','0','0','0','0','now','0','now');
INSERT INTO views VALUES('1','Default View','Default View');

INSERT INTO templates VALUES('0','0','Default Template','Default Template');
INSERT INTO templates VALUES('1','1','Default Owner Template','Default Owner Template');
INSERT INTO templates VALUES('2','2','Default Location Template','Default Location Template');
INSERT INTO templates VALUES('3','3','Default Device Template','Default Device Template');
INSERT INTO templates VALUES('4','4','Default Network Template','Default Network Template');
INSERT INTO templates VALUES('5','5','Default Address Template','Default Address Template');
INSERT INTO regex VALUES('1','Email Address','xxx@yyy.zzz','([\\w\\-\\.\\+]+)\\@([\\w\\-\\.]+)\\.(\\w{3}|\\w{2})');
INSERT INTO regex VALUES('2','Phone Number','111-222-3333','^[\\d\\-\\(\\)\\ ]+$');
INSERT INTO regex VALUES('3','State','XX','([.\\w{2}])');
INSERT INTO regex VALUES('4','Zip Code','55555','([.\\d{5}])');
INSERT INTO regex VALUES('5','Zip Code Long','55555-4444','([.\\d{5}]\\-[.\\d{4}])');
INSERT INTO regex VALUES('6','Zip Code Both','55555, 55555-4444','^(\\d{5}\\-\\d{4}|\\d{5})$');
INSERT INTO regex VALUES('7','Alphanumeric','A-Z, a-z, 0-9, .#&-@,','^[\\d\\w\\ \\.\\#\\&\\,\\\'\\-\\n\\@\\,\\W]+$');
INSERT INTO regex VALUES('8','Number','0-9','^\\d+$');
INSERT INTO regex VALUES('9','Date','mm-dd-yy','^(\\d{1,2}\\/\\d{1,2}\\/\\d{2})$');
INSERT INTO regex VALUES('10','IPv4 Address','255.255.255.255','^[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+$');
INSERT INTO regex VALUES('11','CIDR Address','10.0.0.0/8','^[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\/\\d+$');
INSERT INTO regex VALUES('12','MAC Address','00:22:55:aa:bb:ff','^[0-9a-fA-F]{2}\\:[0-9a-fA-F]{2}\\:[0-9a-fA-F]{2}\\:[0-9a-fA-F]{2}\\:[0-9a-fA-F]{2}\\:[0-9a-fA-F]{2}$');
INSERT INTO regex VALUES('13','Anything','anything','.*');
INSERT INTO regex VALUES('14','Postal Code (Canada)','X1X1X1, X1X 1X1','^([a-zA-Z]\\d[a-zA-Z]\\d[a-zA-Z]\\d|[a-zA-Z]\\d[a-zA-Z]\\ \\d[a-zA-Z]\\d)$');
INSERT INTO fielddefs VALUES('1','5','MAC Address','MAC Address of the interface using this address','F','12','0','20','1','TEXT','','2');
INSERT INTO fielddefs VALUES('2','1','Street Address','Street Address of the Owner','T','7','0','100','1','TEXT','','2');
INSERT INTO fielddefs VALUES('3','1','City','City of the Owner','T','7','0','100','2','TEXT','','2');
INSERT INTO fielddefs VALUES('4','1','State','State of the Owner','F','3','0','50','3','TEXT','','2');
INSERT INTO fielddefs VALUES('5','1','Zip/Postal Code','Zip Code of the Owner','T','6','0','50','4','TEXT','','2');
INSERT INTO fielddefs VALUES('6','1','POC Name','Point of Contact Name','T','7','0','50','5','TEXT','','2');
INSERT INTO fielddefs VALUES('7','1','POC Email','Point of Contact Email','T','1','0','50','6','TEXT','','2');
INSERT INTO fielddefs VALUES('8','1','POC Phone','Point of Contact Phone','T','2','0','50','7','TEXT','','2');
INSERT INTO fielddefs VALUES('9','1','POC Pager','Point of Contact Pager','F','2','0','50','8','TEXT','','2');
INSERT INTO fielddefs VALUES('10','1','POC Cell','Point of Contact Cell','F','2','0','50','9','TEXT','','2');
INSERT INTO fielddefs VALUES('11','1','Tech Name','Tech Point of Contact Name','F','7','0','50','10','TEXT','','2');
INSERT INTO fielddefs VALUES('12','1','Tech Email','Tech Point of Contact Email','F','1','0','50','11','TEXT','','2');
INSERT INTO fielddefs VALUES('13','1','Tech Phone','Tech Point of Contact Phone','F','2','0','50','12','TEXT','','2');
INSERT INTO fielddefs VALUES('14','1','Tech Pager','Tech Point of Contact Pager','F','2','0','50','13','TEXT','','2');
INSERT INTO fielddefs VALUES('15','1','Tech Cell','Tech Point of Contact Cell','F','2','0','50','14','TEXT','','2');