#!/usr/bin/perl
package NorthStarUpdater;

# 
# Copyright (C) 2001, Hitesh Patel <hitesh@presys.com>
# 
# This file is part of NorthStar.
#
# NorthStar is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# NorthStar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with NorthStar; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#

use DBI;
use Net::IP;

$VERSION = "1.25";
my(@searchdir) = qw(/opt/NorthStar /usr/NorthStar /usr/local/NorthStar);
my($t);

foreach(@searchdir) {
    if(-f "$_/NorthStar.conf") {
        ConfigInit("$_/NorthStar.conf");
        $t = 1;
        last;
    }
}

if(!$t) {
    die("Unable to find the configuration file.\n");
}

my(%config) = ConfigGetSection('Global');
my(%dbconfig) = ConfigGetSection('Database');
my($lib_dir) = $config{'InstallDir'} . "/lib/";

require $lib_dir . 'dbutil.lib';

my(%dbinfo) = (
  'fh'      => 'db',		              # Database FH, DONT change this
  'driver'	=> $dbconfig{'DatabaseDriver'},# DBI database driver
  'host'    => $dbconfig{'DatabaseHost'}, # The host address of your db server
  'port'    => $dbconfig{'DatabasePort'}, # Database port
  'options' => '',		                  # Unused
  'tty'     => '',		                  # Unused
  'dbname'  => $dbconfig{'DatabaseName'}, # The database to connect to
  'dbuser'  => $dbconfig{'DatabaseUser'}, # The user to connect as
  'dbpass'  => $dbconfig{'DatabasePass'}  # The password to use
);

if(db_connect(\%dbinfo) != 0) {
	ExitError("db_connect: " . $DBI::errstr);
}

main();

sub main {
    my($c, $sth);

    print << 'EOF';






Welcome to the NorthStar Database Updater.  This updater
should only be used with version $VERSION of NorthStar.  

Please make sure you have backed up your existing database
before running this program... we wouldn't want you to 
loose all your data would we? ;-)

EOF
    print "Would you like to continue with the update [y/n]? ";
    $c = <STDIN>;
    chop($c);
    lc($c);
    if($c ne 'y') {
        ExitError("Aborting update");
    }


    UpdateNetworks();
    UpdateAddresses();

    print << 'EOF';


Ok, were done.  Give it a run and see how it
works!!

EOF

}

sub UpdateNetworks {
    my($sql, $sth, %nets, @x, @row);

    $sql = "ALTER TABLE networks RENAME TO networks_old";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'rename_old_net_table','1');

    $sql = "DROP INDEX networks_pkey";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'delete_old_net_pkeyindex','1');

    $sql =
"CREATE TABLE networks (".
"	id		INT4 NOT NULL DEFAULT nextval('networks_id_seq') PRIMARY KEY,".
"	parent		INT4 NOT NULL REFERENCES networks(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE DEFAULT '0',".
"	name		VARCHAR(256) NOT NULL,".
"	descr		TEXT,".
"	label		VARCHAR(256) NOT NULL,".
"	network		NUMERIC(40,0) NOT NULL,".
"	masklen		INT4 NOT NULL,".
"	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	device		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	assigndate	TIMESTAMP DEFAULT now(),".
"	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	upddate		TIMESTAMP DEFAULT now()".
");";

    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'create_new_net_table','1');

	$sth = $db->prepare("SELECT * FROM networks_old ORDER BY id");
	$sth->execute;
	db_rescheck(\$sth,'get_old_net_data','1');

	for($i = 0; $i < $sth->rows; $i++) {
		@row = $sth->fetchrow_array;
		$nets{$row[0]}{id} = $row[0];
		$nets{$row[0]}{parent} = $row[1];
		$nets{$row[0]}{name} = $row[2];
		$nets{$row[0]}{descr} = $row[3];
		$nets{$row[0]}{network} = $row[4];
		$nets{$row[0]}{owner} = $row[5];
		$nets{$row[0]}{device} = $row[6];
		$nets{$row[0]}{assignuser} = $row[7];
		$nets{$row[0]}{assigndate} = $row[8];
		$nets{$row[0]}{updateuser} = $row[9];
		$nets{$row[0]}{upddate} = $row[10];
		push(@x, $row[0]);
	}

	foreach(@x) {
		$ip = new Net::IP($nets{$_}{network},4) or ExitError(Net::IP::Error());	
		$nets{$_}{masklen} = $ip->prefixlen || 0;
		$nets{$_}{int} = $ip->intip || 0;
		$r = $nets{$_};
		print "$r->{id} $r->{network} = $r->{int}/$r->{masklen}\n";
		$sth = $db->prepare("INSERT INTO networks VALUES ('$r->{id}','$r->{parent}','$r->{name}','$r->{descr}','$r->{network}','$r->{int}','$r->{masklen}','$r->{owner}','$r->{device}','$r->{assignuser}','$r->{assigndate}','$r->{updateuser}','$r->{upddate}');\n");
		$sth->execute;
		db_rescheck(\$sth,'add_new_net_entry','1');

	}

    $sql = "DROP TABLE networks_old";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'drop_old_net_table','1');

}

sub UpdateAddresses {
    my($sql, $sth, %addr, @x, @row);

    $sql = "ALTER TABLE addresses RENAME TO addresses_old";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'rename_old_addr_table','1');

    $sql = "DROP INDEX addresses_pkey";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'delete_old_addr_pkeyindex','1');

    $sql = 
"CREATE TABLE addresses (".
"	id              INT4 NOT NULL DEFAULT nextval('addresses_id_seq') PRIMARY KEY,".
"	parent		INT4 NOT NULL REFERENCES networks(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	address		VARCHAR(256) NOT NULL,".
"	owner		INT4 NOT NULL REFERENCES owners(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".
"	device		INT4 NOT NULL REFERENCES devices(id) MATCH FULL ON DELETE CASCADE ON UPDATE CASCADE,".    
"	dnsname		VARCHAR(128),".
"	macaddr		VARCHAR(20),".
"	misc		TEXT,".
"	assignuser	INT4 NOT NULL REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	assigndate	TIMESTAMP DEFAULT now(),".
"	updateuser	INT4 REFERENCES users(id) MATCH FULL ON UPDATE CASCADE,".
"	upddate		TIMESTAMP DEFAULT now()".
");";

    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'create_new_net_table','1');
       
    $sth = $db->prepare("SELECT * FROM addresses_old ORDER BY id");
	$sth->execute;
	db_rescheck(\$sth,'get1','1');

	for($i = 0; $i < $sth->rows; $i++) {
		@row = $sth->fetchrow_array;
		$addr{$row[0]}{id} = $row[0];
		$addr{$row[0]}{parent} = $row[1];
		$addr{$row[0]}{address} = $row[2];
		$addr{$row[0]}{owner} = $row[3];
		$addr{$row[0]}{device} = $row[4];
		$addr{$row[0]}{dnsname} = $row[5];
		$addr{$row[0]}{macaddr} = $row[6];
		$addr{$row[0]}{misc} = $row[7];
		$addr{$row[0]}{assignuser} = $row[8];
		$addr{$row[0]}{assigndate} = $row[9];
		$addr{$row[0]}{updateuser} = $row[10];
		$addr{$row[0]}{upddate} = $row[11];
		push(@x, $row[0]);
	}

	foreach(@x) {
		$r = $addr{$_};
		print "$r->{id} $r->{address}\n";
		$sth = $db->prepare("INSERT INTO addresses VALUES ('$r->{id}','$r->{parent}','$r->{address}','$r->{owner}','$r->{device}','$r->{dnsname}','$r->{macaddr}','$r->{misc}','$r->{assignuser}','$r->{assigndate}','$r->{updateuser}','$r->{upddate}');\n");
		$sth->execute;
		db_rescheck(\$sth,'add_new_addr_entry','1');
	}

    $sql = "DROP TABLE addresses_old";
    $sth = $db->prepare($sql);
	$sth->execute();
	db_rescheck(\$sth,'drop_old_addr_table','1');

}





sub ExitError {
    my($msg) = $_[0];
    my($package, $filename, $line) = caller;
    print "\nERROR: $msg at $package:$filename:$line\n";
    exit(1);
}
    
sub ConfigGetSection {
        my $sec = shift;
        my(%t);
        
        my($h) = $NorthStar::CONFIG{$sec};
        while(my($k,$v) = each %$h) {
                $t{$k} = $v;
        }
        return(%t);
}

sub ConfigGetValue {
        if(scalar(@_) == 3) {
                return($NorthStar::CONFIG{$_[0]}{$_[1]}{$_[2]});
        } elsif(scalar(@_) == 2) {
                return($NorthStar::CONFIG{$_[0]}{$_[1]});
        } else {
                return(undef);
        }
}

sub ConfigInit {
        my $cf = shift;
        my($cs, $csv, $cn, $cv, @j, $x);
        open(_CONFIG, "<$cf") || die("config file open failed: $cf: $!\n");
        while(<_CONFIG>) {
                chomp;
                s/^[\s\t]+//g; #strip leading whitespace
                next if /^#/;
                next if /^$/;
                s/\s+$//g; #strip trailing whitespace

                if(/\#/) {
                    /(.*)\#.*/;
                    $x = $1;
                } else {
                    $x = $_;
                }

                # given:
                # <Section SectionVariable>
                #                Name = Value
                # </Section>
                # $cs  = Section                Config Section        
                # $csv = SectionVariable        Config Section Variable 
                # $cn  = Name                   Config Name
                # $cv  = Value                  Config Variable
                # 
                # $csv may not be defined if a SectionVariable was not present
                if($x =~ /^<(.*)>$/ && $x !~ /^<\/.*>/) {           # handler for section starts <Section>
                        if(!defined($cs)) {              # check to see if the previous section was closed with </Section> 
                                $cs = $1;
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "previous configuration section not closed (maybe you forgot </$cs>)\n");
                        }

                        # sub-handler for sections with a variable <Section SectionVariable>
                        if($x =~ /^<(.*) (.*)>$/ && (@j = split(/\s+/, $x)) == 2) {
                                $cs = $1;
                                $csv = $2;
                        } elsif($x =~ /^<(.*)>$/) { # dont clobber sections with no SectionVariable (<Section> ..)
                                $cs = $1;
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "configuration sections may only have one parameter (<Section Parameter>)\n");
                        }
                        next;
                } elsif($x =~ /^<\/(.*)>$/) {            #handler for section closes </Section>
                        if(defined($cs)) {         #check to see if this section was ever opened
                                undef($cs);
                                undef($csv);
                        } else {
                                $cs = $1;          #set section name for error message
                                $cs =~ s/\///g;    #strip leading '/'
                                die("syntax error in configuration file \"$cf\" at line $.: " .
                                    "configuration section not opened (maybe you forgot <$cs>)\n");
                        }
                        next;
                } else {
                        if($x =~ /(.*) \"(.*)\"/) {               #process line "Name = "Value In Here"
                                $cn = $1;                                
                                $cv = $2;
                                $cv =~ s/\\(.)/$1/g;              #normalize escaped charecters like: \"
                        } elsif((@j = split(/\s+/, $x)) > 2) {    #process line "Name = Value1 Value2..."
                                $cn = shift(@j);
                                $cv = join(',', @j);              #make a comma delimited list
                        } elsif((@j = split(/\s+/, $x)) == 2) {   #process line "Name = Value"
                                $cn = $j[0];
                                $cv = $j[1];
                        } else {
                                die("syntax error in configuration file \"$cf\" at line $.\n");
                        }

                        if($csv) { #if SectionVariable was set this as CONFIG->Section->SectionVariable->Name=Value
                                #$self->Debug("add value: CONFIG->$cs->$csv->$cn=$cv");
                                $NorthStar::CONFIG{$cs}{$csv}{$cn} = $cv;
                        } else {   #else set this value under CONFIG->Section->Name=Value
                                #$self->Debug("add value: CONFIG->$cs->$cn=$cv");
                                $NorthStar::CONFIG{$cs}{$cn} = $cv;
                        }
                        
                        next;
                }
                
                
        }
        #$self->Debug("closing config file");
        close(_CONFIG) || die("close failed: $cf: $!\n");

        return 1;
}

sub ConfigPrint {
        if(!defined(%NorthStar::CONFIG)) {
                return;
        }
        
        while(my($k,$v) = each %NorthStar::CONFIG) {
                print STDERR "$k\n";
                while(my($y,$z) = each %$v) {
                        if(ref($z) eq "HASH") {
                                print STDERR " $y\n";
                                while(my($a,$b) = each %$z) {
                                        print STDERR "  $a=$b\n";
                                }
                        } else {
                                print STDERR " $y=$z\n";
                        }
                }
                print STDERR "\n";
        }
}
